# Copyright 2022-present Lenovo
# Confidential and Proprietary

import os
import json

from dataset_analysis import DatasetAnalysisBase


class DatasetAnalysis(DatasetAnalysisBase):

    ACCEPTABLE_FILE_PATTERNS = ['.jpg', '.png', '.jpeg', '.bmp', '.gif']

    def verify_format(self, dataset_path):
        dirs_list = ['training', 'validation']
        files_list = ['training/training.csv', 'validation/validation.csv']

        self.check_structure(dataset_path, dirs_list, files_list)

        sample_num = 0
        for d in dirs_list:
            sample_num += self.get_sample_num(
                os.path.join(dataset_path, d),
                self.ACCEPTABLE_FILE_PATTERNS
            )

        self.check_empty_dataset(dataset_path, sample_num)
        size = self.get_dir_size_b(dataset_path)
        return {
            "size_b": size,
            "label_num": -1,
            "sample_num": sample_num,
            "label_statistics": json.dumps({
                "Size": size,
                "Samples": sample_num
            })
        }
