# Copyright 2022-present Lenovo
# Confidential and Proprietary

import os.path
import json

from dataset_analysis import DatasetAnalysisBase


class DatasetAnalysis(DatasetAnalysisBase):
    ACCEPTABLE_FILE_PATTERNS = ['.jpg', '.png', '.jpeg', '.bmp', '.gif']

    def verify_format(self, dataset_path):
        dirlist = []
        if self.ds_format == 'default':
            dirlist = ["train", "val"]
        elif self.ds_format == 'coco_1.0':
            dirlist = ["images"]
        else:
            print(f"[ERROR]: invalid instance segmentation dataset "
                  f"format {self.ds_format}")
            exit(1)

        filelist = ["annotations/instances_train.json",
                    "annotations/instances_val.json"]
        self.check_structure(dataset_path, dirlist, filelist)

        sample_num = 0
        for d in dirlist:
            sample_num += self.get_sample_num(
                os.path.join(dataset_path, d),
                self.ACCEPTABLE_FILE_PATTERNS
            )

        self.check_empty_dataset(dataset_path, sample_num)
        size = self.get_dir_size_b(dataset_path)
        return {
            "size_b": size,
            "label_num": -1,
            "sample_num": sample_num,
            "label_statistics": json.dumps({
                "Size": size,
                "Samples": sample_num
            })
        }
