# Copyright 2022-present Lenovo
# Confidential and Proprietary

import os
import json
from glob import glob

from dataset_analysis import DatasetAnalysisBase


class DatasetAnalysis(DatasetAnalysisBase):
    def verify_format(self, dataset_path):
        dirlist = [
            'training', 'validation',
            'training/image', 'training/label',
            'validation/image', 'validation/label'
        ]
        self.check_structure(dataset_path, dirlist, [])

        sample_num = 0
        label_num = 0
        for p in ('training', 'validation'):

            input_fname_pattern = self.get_file_type(
                os.path.join(dataset_path, p))
            if len(input_fname_pattern) < 3:
                print('[ERROR]: There are no valid image files '
                      'in {}'.format(dataset_path))
                exit(1)
            image_arr = glob(os.path.join(
                dataset_path, p, 'image', input_fname_pattern))
            images = [os.path.basename(image_path) for image_path in image_arr]
            labels = os.listdir(os.path.join(dataset_path, p, 'label'))

            if not set(images).issubset(set(labels)):
                labeled_image_set = set(images).intersection(set(labels))
                no_labeled_image_set = set(images) - labeled_image_set
                print('[WARNING]: Images cannot find correspond labels: {0}'
                      .format(no_labeled_image_set))

            sample_num += len(images)
            label_num += len(labels)
            # images may be less than labels, only count valid labels

        self.check_empty_dataset(dataset_path, sample_num)
        size = self.get_dir_size_b(dataset_path)
        return {
            "size_b": size,
            "label_num": label_num,
            "sample_num": sample_num,
            "label_statistics": json.dumps({
                "Size": size,
                "Samples": sample_num
            })
        }
