# Copyright 2022-present Lenovo
# Confidential and Proprietary

import os.path
import json

from dataset_analysis import DatasetAnalysisBase


class DatasetAnalysis(DatasetAnalysisBase):

    ACCEPTABLE_FILE_PATTERNS = ['.jpg', '.png', '.jpeg', '.bmp', '.gif']

    def verify_format(self, dataset_path):
        label_num = -1
        sample_num = 0

        if self.ds_format in ('default', 'voc_1.1'):
            annotation_folder = 'Annotations'
            category_file = 'category.txt'
            dirlist = [annotation_folder, 'JPEGImages']
            filelist = ['ImageSets/Main/train.txt',
                        'ImageSets/Main/val.txt',
                        category_file]
            self.check_structure(dataset_path, dirlist, filelist)

            label_num, category_list = self.verify_format_category(
                os.path.join(dataset_path, category_file))

            sample_num = self.get_sample_num(
                os.path.join(dataset_path, 'JPEGImages'),
                self.ACCEPTABLE_FILE_PATTERNS
            )

        elif self.ds_format == 'autodl':
            dirlist = ['train/images', 'val/images']
            filelist = ['train/Annotations.zip', 'val/Annotations.zip']
            self.check_structure(dataset_path, dirlist, filelist)

            for d in dirlist:
                sample_num += self.get_sample_num(
                    os.path.join(dataset_path, d),
                    self.ACCEPTABLE_FILE_PATTERNS
                )
        else:
            print(f"[ERROR]: invalid object detection dataset "
                  f"format {self.ds_format}")
            exit(1)

        self.check_empty_dataset(dataset_path, sample_num)
        size = self.get_dir_size_b(dataset_path)
        return {
            "size_b": size,
            "label_num": label_num,
            "sample_num": sample_num,
            "label_statistics": json.dumps({
                "Size": size,
                "Samples": sample_num
            })
        }

    def verify_format_category(self, category_path):
        label_num = 0
        with open(category_path, 'r', encoding='utf-8') as f:
            category_list = f.read().splitlines()
            if category_list[0] != '__background__':
                print("[Error]: First line of the category file "
                      "is not '__background__'")
                exit(1)
            label_num = len(category_list) - 1
        return label_num, category_list
