# Copyright 2022-present Lenovo
# Confidential and Proprietary

import os.path
import json

from dataset_analysis import DatasetAnalysisBase


class DatasetAnalysis(DatasetAnalysisBase):
    def verify_format(self, dataset_path):
        dirlist = ["training", "validation"]
        self.check_structure(dataset_path, dirlist, [])

        sample_num = 0
        for d in dirlist:
            data_dir = os.path.join(dataset_path, d)
            for filename in os.listdir(data_dir):
                file_path = os.path.join(data_dir, filename)
                if os.path.isfile(file_path) and 'train_from' in filename:
                    sample_num += self.get_line_count(file_path)

        self.check_empty_dataset(dataset_path, sample_num)
        size = self.get_dir_size_b(dataset_path)
        return {
            "size_b": size,
            "label_num": -1,
            "sample_num": sample_num,
            "label_statistics": json.dumps({
                "Size": size,
                "Samples": sample_num
            })
        }

    def get_line_count(self, file_path):
        count = 0
        try:
            fd = open(file_path)
            while True:
                buffer = fd.read(1024 * 8192)
                if not buffer:
                    break
                count += buffer.count('\n')
        except Exception:
            print(f"[ERROR]: Failed to get line count of file {file_path}")
            exit(1)
        finally:
            fd.close()
        return count
