# Copyright 2022-present Lenovo
# Confidential and Proprietary

import os.path
from collections import defaultdict
import json

from dataset_analysis import DatasetAnalysisBase


class DatasetAnalysis(DatasetAnalysisBase):
    def verify_format(self, dataset_path):
        files_list = ['training.tsv', 'validation.tsv']
        self.check_structure(dataset_path, [], files_list)

        counts = defaultdict(int)
        total_num = 0
        counts_list = []
        for files in files_list:
            files_path = os.path.join(dataset_path, files)
            samp_num = self.get_parameter_values(files_path, counts)
            total_num += samp_num
            counts_list.append(len(counts))
        if counts_list[0] < counts_list[1]:
            print('[Error]: Label content is out of scope in the '
                  'validation.tsv file.')
            exit(1)

        self.check_empty_dataset(dataset_path, total_num)
        size = self.get_dir_size_b(dataset_path)
        return {
            "size_b": size,
            "label_num": len(counts),
            "sample_num": total_num,
            "label_statistics": json.dumps({
                "Size": size,
                "Samples": total_num
            })
        }

    def get_parameter_values(self, filename_path, counts):
        try:
            with open(filename_path, 'r', encoding='utf-8') as f:
                total_num = 0
                for line in f:
                    re_result = line.split('\t', 1)
                    labels = re_result[0]
                    counts[labels] += 1
                    total_num += 1
        except Exception:
            print('[Error]: The {} format is wrong.'.format(
                os.path.basename(filename_path)))
            exit(1)
        return total_num
