# Copyright 2018-present Lenovo
# Confidential and Proprietary

import os

AUTOML_MODEL_NAME = ["large", "big", "standard", "small", "mini", "tiny"]


def get_infe_serv(scenario, model_path, model_name, **kwargs):  # noqa: C901
    print('Init Load Model')

    plan_file = None
    for filepath, dirnames, filenames in os.walk(model_path):
        if plan_file:
            break
        for filename in filenames:
            if filename.endswith('.engine'):
                plan_file = os.path.join(filepath, filename)
                break

    if plan_file:
        from trt_infe_serv import TRTDeploySer
        inference_service_mgt = TRTDeploySer()
    elif scenario == "classification":
        if model_name in AUTOML_MODEL_NAME:
            from autoML_classification_infe_serv import \
                AutoClassificationDeploySer
            inference_service_mgt = AutoClassificationDeploySer()
        else:
            from classification_infe_serv import ClassificationDeploySer
            inference_service_mgt = ClassificationDeploySer()
    elif scenario == "objectdetection":
        if model_name in AUTOML_MODEL_NAME:
            from autoML_detection_infe_serv import AutoDetectionDeploySer
            inference_service_mgt = AutoDetectionDeploySer()
        elif model_name == "yolov3":
            from yolov3_detection_infe_serv import DetectionDeploySer
            inference_service_mgt = DetectionDeploySer()
        else:
            from fastrcnn_detection_infe_serv import DetectionDeploySer
            inference_service_mgt = DetectionDeploySer()
    elif scenario == "segmentation":
        from segmentation_infe_serv import SegmentationDeploySer
        inference_service_mgt = SegmentationDeploySer()
    elif scenario == "text":
        from text_infe_serv import TextDeploySer
        inference_service_mgt = TextDeploySer(**kwargs)
    else:
        message = 'The scenario must be one of classification,' + \
                  ',objectdetection, segmentation. But get' + scenario
        raise Exception(message)
    inference_service_mgt.load_model(model_path, model_name)
    print('Load Model Success')
    return inference_service_mgt
