# Copyright 2021-present Lenovo
# Confidential and Proprietary

import json
import os
import sys

import numpy as np
import tensorflow.compat.v1 as tf
tf.disable_v2_behavior()
from tensorflow.python.platform import gfile

parentpath = '/opt/letrain/'
path_list = [parentpath,
             parentpath + 'lib',
             parentpath + 'engine',
             parentpath + 'engine/automl/evaluator',
             parentpath + 'tfserving',
             parentpath + 'inference',
             parentpath + 'applications',
             os.path.join(parentpath, "applications/frcnn")]

sys.path += path_list

from engine.automl.learner.modelset.tfefficientnet.backbone import (
    backbone_factory, efficientnet_model
)
from engine.automl.learner.modelset.tfefficientnet.backbone.\
    efficientnet_builder import BlockDecoder
from engine.automl.learner.modelset.tfefficientnet.backbone.\
    efficientnet_builder import efficientnet as tfefficientnet

from engine.automl.learner.modelset.tfefficientdet.utils import \
    BatchNormalization
from engine.preprocessing.image_preprocessing import preprocess_image


sys.path += ['/opt/letrain/applications/frcnn/lib/utils/']
sys.path += ["/opt/letrain/engine/automl/learner/modelset/tfefficientnet/"]

from autoML_common import restore_ckpt
from timer import Timer

tf.app.flags.DEFINE_bool(
    'automl_train', False, 'Select whether to make the inference.'
)
tf.app.flags.DEFINE_bool(
    'test_transfer', True, 'Select whether to make the inference.'
)
tf.app.flags.DEFINE_string("output_dir", "", "")
tf.app.flags.DEFINE_integer("image_size", None, "")
tf.app.flags.DEFINE_string('debug', 'No', '')

FLAGS = tf.app.flags.FLAGS


class AutoClassificationDeploySer(object):

    def get_preprocess_fn(self):
        """Build input dataset."""
        return preprocess_image

    def build_dataset(self, filenames, labels, is_training):
        """Build input dataset."""
        batch_drop_remainder = False
        filenames = tf.constant(filenames)
        labels = tf.constant(labels)
        dataset = tf.data.Dataset.from_tensor_slices((filenames, labels))
        # amend
        if self.image_size is None:
            self.image_size = 224

        def _parse_function(filename, label):
            image_string = tf.read_file(filename)
            image_decoded = tf.cond(
                tf.image.is_jpeg(image_string),
                lambda: tf.image.decode_jpeg(image_string, channels=3),
                lambda: tf.image.decode_png(image_string, channels=3))

            image_decoded = tf.image.convert_image_dtype(
                image_decoded, tf.float32)
            image_decoded = tf.image.resize_images(
                image_decoded, size=(self.image_size, self.image_size))
            image_decoded.set_shape([self.image_size, self.image_size, 3])
            # image_final = tf.image.per_image_standardization(image_decoded)
            # label = tf.one_hot(label,self.class_num)
            preprocess_fn = self.get_preprocess_fn()
            # if FLAGS.do_autoaugment:
            do_autoaugment = True
            if do_autoaugment:
                size = self.image_size
                resize = int(size * 1.15)
                image_decoded = preprocess_fn(
                    image_decoded, size, size, resize, is_training=False)
            return image_decoded, label

        dataset = dataset.map(_parse_function)
        dataset = dataset.batch(
            self.batch_size, drop_remainder=batch_drop_remainder)

        iterator = dataset.make_one_shot_iterator()
        images, labels = iterator.get_next()
        return images, labels

    # test amend
    def process_lable_map(self, labels_map_file):
        label_txt = [i.strip() for i in open(labels_map_file, "r").readlines()]
        list_json = {}
        for i in label_txt:
            key, value = i.split(":")
            list_json[key] = value
        label_map_path = os.path.join(
            os.path.dirname(labels_map_file), "labels_map.txt")
        if os.path.isfile(label_map_path):
            os.remove(label_map_path)
        with open(label_map_path, "w") as f:
            f.write(json.dumps(list_json, indent=2, ensure_ascii=False))
        return label_map_path

    def load_model(self, model_path, model_name):
        "pre check all flags to make sure following work can be executed."
        self.LABELS_FILE = model_path + '/labels.txt'

        self.model_name = None
        try:
            with open(os.path.join(model_path, "fix_gear.txt"), 'r') as f:
                fix_gear = f.readlines()[0]
                self.model_name = "efficientnet-b" + str(int(fix_gear) - 1)
        except Exception as e:
            print("Failed to get fix gear. \n{0}".format(e), flush=True)

        self.pre_check(model_path)

        if not FLAGS.output_dir:
            FLAGS.output_dir = os.path.join(model_path, "infer_out_dir")
        if FLAGS.image_size:
            self.image_size = FLAGS.image_size
        else:
            self.image_size = 224
        self.labels_map_file = self.process_lable_map(
            os.path.join(model_path, "labels.txt"))
        self.classes = json.loads(tf.gfile.Open(
            self.labels_map_file).read())

        self.MODEL_DIR = model_path
        self.sess = tf.Session(
            config=tf.ConfigProto(allow_soft_placement=True))
        init = tf.global_variables_initializer()
        self.sess.run(init)

        self.input_image = tf.placeholder(
            tf.float32, [1, self.image_size, self.image_size, 3])
        probs = self.build_model(self.input_image, is_training=False)
        if isinstance(probs, tuple):
            self.probs = probs[0]
        else:
            self.probs = probs
        restore_ckpt(self.sess, self.MODEL_DIR, False, None)

    def inference(self, image_file_name):
        image_files = [image_file_name]
        labels = [0] * len(image_files)
        self.batch_size = 1

        # Code below are used for actual op compute
        timer = Timer()
        timer.tic()

        images, labels = self.build_dataset(image_files, labels, False)
        prediction_idx = []
        prediction_prob = []
        for _ in range(len(image_files) // self.batch_size):
            out_probs = self.sess.run(
                self.probs,
                feed_dict={self.input_image: self.sess.run(images)})
            idx = np.argsort(out_probs)[::-1]
            prediction_idx.append(idx[:5])
            prediction_prob.append([out_probs[pid] for pid in idx[:5]])

        timer.toc()
        classification_result = {}
        for i in range(len(image_files)):
            classification_data = []
            for j, idx in enumerate(prediction_idx[i]):
                classification_data.append({
                    'label': self.classes[str(idx)],
                    'probability': round(float(prediction_prob[i][j]), 4)})
        # parse the classification infernece results
        classification_result = {
            "data": classification_data,
            "duration": int(round(1000 * timer.total_time))}

        return classification_result

    def pre_check(self, model_path):
        if not self.model_name:
            raise ValueError("Inference model name must be provided.")
        if not model_path:
            raise ValueError(
                "Checkpoint dir must be provided to restore model."
            )
        if not gfile.Exists(self.LABELS_FILE):
            raise ValueError("label file: %s is not found." % self.LABELS_FILE)

    def build_model(self, features, is_training):
        """Build model with input features."""
        num_classes = len(open(self.LABELS_FILE, "r").readlines())
        if os.path.exists(os.path.join(self.MODEL_DIR, "model_scaling.json")):
            f = open(os.path.join(self.MODEL_DIR, "model_scaling.json"), "r")
            result = json.load(f)
            width = result["width"]
            depth = result["depth"]
            resolution = result["resolution"]
            model_name = result["model_name"]
            self.image_size = resolution
            dropout_rate = self.get_droprate(int(resolution * 224))
            global_params = tfefficientnet(width, depth, dropout_rate)
            global_params = global_params._replace(
                **{'num_classes': num_classes})
            decoder = BlockDecoder()
            blocks_args = decoder.decode(global_params.blocks_args)
            if float(tf.__version__[:1]) >= 2:
                with tf.compat.v1.variable_scope(
                        model_name, reuse=tf.compat.v1.AUTO_REUSE):
                    model = efficientnet_model.Model_new(
                        blocks_args, global_params)
                    logits = model(features, training=False,
                                   features_only=False)
            else:
                with tf.variable_scope(model_name, reuse=tf.AUTO_REUSE):
                    model = efficientnet_model.Model_new(
                        blocks_args, global_params)
                    logits = model(features, training=False,
                                   features_only=False)
        else:
            model_builder = backbone_factory.get_model_builder(self.model_name)
            override_params = {}
            override_params['num_classes'] = num_classes
            override_params['batch_norm'] = BatchNormalization
            _, logits = model_builder.build_model_base_cv(
                features, self.model_name, False,
                override_params=override_params)
        probs = tf.nn.softmax(logits)
        probs = tf.squeeze(probs)
        return probs

    def get_droprate(self, resolution):
        rate = 0
        if resolution < 260:
            rate = 0.2
        elif resolution < 380:
            rate = 0.3
        elif resolution < 500:
            rate = 0.4
        else:
            rate = 0.5
        return rate


if __name__ == '__main__':
    model_path = "/home/hpcadmin/ai_studio/trained_models/94/model"
    model_name = "efficientnet-b2"
    image_file_name = "/home/hpcadmin/.cache/7d95353c-5fb4-4b8e-8d71-" \
                      "06ac0f157e75/input/7d95353c-5fb4-4b8e-8d71-06ac0f157e75"
    detection_deploy_ser = AutoClassificationDeploySer()
    detection_deploy_ser.load_model(model_path, model_name)
    print(detection_deploy_ser.inference(image_file_name))
