# Copyright 2018-present Lenovo
# Confidential and Proprietary

import base64
import os
import sys
import traceback
from tempfile import NamedTemporaryFile

import falcon
from adapter import get_infe_serv
from falcon.media.validators import jsonschema


class DeployServiceRunner(object):
    def __init__(self, scenario, deploy_path, model_name, **kwargs):
        self.deploy_path = deploy_path
        self.model_path = os.path.join(deploy_path, 'model')
        self.InferenceServiceMgt = get_infe_serv(
            scenario, self.model_path, model_name, **kwargs
        )
        sys.stdout.flush()

    @jsonschema.validate(
        {
            "type": "object",
            "properties": {
                "image": {"type": "string"},
                "text_content": {"type": "string"},
                "with_original_content": {"type": "boolean"},
                "input_type": {
                    "type": "string",
                    "enum": ["BASE64"]
                },
            },
            "dependencies": {
                "image": ["input_type"]
            }
        }
    )
    def on_post(self, req, resp):
        try:
            with_original_content = req.media.get('with_original_content')
            if req.media.get('image'):
                flag = 'image'
                content = base64.b64decode(req.media.get('image'))
            elif req.media.get('text_content'):
                flag = 'text_content'
                content = base64.b64decode(req.media.get('text_content'))

            # tempfile = NamedTemporaryFile()

            with NamedTemporaryFile(mode='wb') as buf:
                buf.write(content)
                buf.flush()
                if with_original_content in [False, True]:
                    inf_result = self.InferenceServiceMgt.inference(
                        buf.name, with_original_content)
                else:
                    inf_result = self.InferenceServiceMgt.inference(buf.name)
        except ValueError:
            traceback.print_exc()
            raise falcon.HTTPBadRequest(
                description='The format of %s is unsupported' % flag
            )
        except TypeError:
            traceback.print_exc()
            raise falcon.HTTPBadRequest(
                description='It is not an %s' % flag
            )
        except Exception:
            traceback.print_exc()
            raise falcon.HTTP_INTERNAL_SERVER_ERROR(
                description='Internal error of inference server'
            )
        resp.media = inf_result
