# Copyright 2018-present Lenovo
# Confidential and Proprietary

import json
import os
import sys
import time

import tensorflow.compat.v1 as tf
tf.disable_v2_behavior()
from adapter import get_infe_serv


def run_inference(scenario, model_path, model_name,
                  image_file, output_dir, **kwargs):
    error_file = os.path.join(output_dir, 'error.json')
    try:
        inference_service_mgt = get_infe_serv(scenario, model_path,
                                              model_name, **kwargs)
        print("Deal with file {}".format(image_file))
    except Exception as e:
        with open(error_file, 'w', encoding="utf-8") as f:
            json.dump({"err_type": "load_model", "err_msg": "%s" % e}, f)
        raise e

    try:
        inf_result = inference_service_mgt.inference(image_file)

        temp_file = os.path.join(
            output_dir, 'temp_file_{}.json'.format(int(time.time()))
        )
        with open(temp_file, 'w') as f:
            inf_result['filename'] = image_file
            f.write(json.dumps(inf_result, indent=4))
            print("{0}".format(inf_result))
        response_file = os.path.join(output_dir, 'result.json')
        os.rename(temp_file, response_file)
        print("Save inference result to {}".format(response_file))
    except Exception as e:
        with open(error_file, "w", encoding="utf-8") as f:
            json.dump({"err_type": "inference", "err_msg": "%s" % e}, f)
        raise e

    sys.stdout.flush()


if __name__ == '__main__':
    tf.app.flags.DEFINE_enum(
        'scenario', 'classification',
        ['classification', 'objectdetection', 'segmentation', 'text'],
        "Scenario for this service"
    )
    tf.app.flags.DEFINE_string(
        'lico_model_path', '', "model path, for inference job"
    )
    tf.app.flags.DEFINE_string(
        'lico_image_file', '', "image file to deal, for inference job"
    )
    tf.app.flags.DEFINE_string(
        'lico_output_dir', '', "output dir, for inference job"
    )
    tf.app.flags.DEFINE_string('lico_model_name', '',
                               "model_name for inference job")
    tf.app.flags.DEFINE_boolean('lico_text_lower_case', True,
                                "text for do_lower_case")
    tf.app.flags.DEFINE_enum('lico_text_language', 'en',
                             ['en', 'zh'], 'text for language')
    tf.app.flags.DEFINE_string("lico_text_content", "", "text content")
    tf.app.flags.DEFINE_integer('lico_text_max_seq_length', 256,
                                "text for max_seq_length")
    FLAGS = tf.app.flags.FLAGS

    run_inference(
        FLAGS.scenario,
        FLAGS.lico_model_path,
        FLAGS.lico_model_name,
        FLAGS.lico_image_file,
        FLAGS.lico_output_dir
    )
