# Copyright 2022-present Lenovo
# Confidential and Proprietary

import base64
import os
import sys

import numpy as np
import tensorflow.compat.v1 as tf
tf.disable_v2_behavior()

letrain_path = "/opt/letrain"
path_list = [os.path.join(letrain_path, 'applications'),
             os.path.join(letrain_path, 'applications/bert_qc'),
             letrain_path,
             letrain_path + 'lib',
             letrain_path + 'engine',
             letrain_path + 'inference',
             letrain_path + 'applications',
             os.path.join(letrain_path, "applications/bert_qc"),
             os.path.join(letrain_path, "applications/frcnn/lib/utils"),
             os.path.join(letrain_path, "inference/bert_qc"),
             os.path.join(letrain_path, "iapplications"),
             os.path.join(letrain_path, "iapplications/bert_qc")]
sys.path += path_list
print(sys.path)

# ----------------------------------------------------------

from tempfile import NamedTemporaryFile

import letrain  # noqa: F401
from core.get_examples import (ColaProcessor, MnliProcessor, MrpcProcessor,
                               UniformProcessor, XnliProcessor)
from core.modeling import BertConfig, BertModel
from get_bert_data import read_verify_fn
from timer import Timer

import tf_slim as slim  # noqa: F401
FLAGS = tf.app.flags.FLAGS


class TextDeploySer(object):
    def __init__(self, **kwargs):
        self.temp_file = NamedTemporaryFile()
        try:
            # There is an error when the content does not exist
            # and you need to catch it
            if kwargs.get("content", None):
                content = kwargs.get("content")
            else:
                content = FLAGS.lico_text_content
            if content:
                content = base64.b64decode(content).decode("utf-8")
                with open(self.temp_file.name, 'w', encoding='utf-8') as buf:
                    buf.write(content)
        except Exception:
            pass
        if kwargs.get("language", None):
            # Use it in deploy
            language = kwargs.get("language")
            FLAGS.do_lower_case = True if kwargs.get(
                "do_lower_case") == 'True' else False
            FLAGS.max_seq_length = int(kwargs.get("max_seq_length"))
        else:
            # Use it in test
            language = FLAGS.lico_text_language
            FLAGS.do_lower_case = FLAGS.lico_text_lower_case
            FLAGS.max_seq_length = int(FLAGS.lico_text_max_seq_length)
        FLAGS.vocab_file = "/opt/pretrain_models/bert_qc/{}/vocab.txt" \
            .format(language)
        FLAGS.bert_config_file = "/opt/pretrain_models/bert_qc/{}/" \
                                 "bert_config.json".format(language)

    def load_model(self, model_path, model_name):
        FLAGS.model_name = model_name
        FLAGS.checkpoint_dir = model_path
        FLAGS.is_onetest = True
        FLAGS.inference = True
        FLAGS.batch_size = 1

    def inference(self, input_file, with_original_content=True):  # noqa: C901
        with tf.Graph().as_default():
            line_num = 1
            if input_file:
                FLAGS.input_file = input_file
            else:
                FLAGS.input_file = self.temp_file.name
            data_direct, _ = read_verify_fn()

            # need data_direct
            bert_config = BertConfig.from_json_file(FLAGS.bert_config_file)
            segment_ids = data_direct[0]
            label_ids = data_direct[1]
            input_mask = data_direct[2]
            input_ids = data_direct[3]

            is_training = False  # ???

            processors = {
                "cola": ColaProcessor,
                "mnli": MnliProcessor,
                "mrpc": MrpcProcessor,
                "xnli": XnliProcessor,
                "uniform": UniformProcessor,
            }

            task_name = FLAGS.task_name.lower()
            processor = processors[task_name]()
            label_list = processor.read_labels(FLAGS.checkpoint_dir)
            num_labels = len(label_list)

            model = BertModel(
                config=bert_config,
                is_training=is_training,
                input_ids=input_ids,
                input_mask=input_mask,
                token_type_ids=segment_ids,
                use_one_hot_embeddings=False)

            output_layer = model.get_pooled_output()

            hidden_size = output_layer.shape[-1].value

            output_weights = tf.get_variable(
                "output_weights", [num_labels, hidden_size],
                initializer=tf.truncated_normal_initializer(stddev=0.02))
            output_bias = tf.get_variable(
                "output_bias", [num_labels],
                initializer=tf.zeros_initializer())

            output_layer = tf.nn.dropout(output_layer, keep_prob=0.9)

            logits = tf.matmul(output_layer, output_weights, transpose_b=True)
            logits = tf.nn.bias_add(logits, output_bias)
            log_probs = tf.nn.softmax(logits, axis=-1)

            one_hot_labels = tf.one_hot(label_ids, depth=num_labels,
                                        dtype=tf.float32)

            # run different
            gpu_options = tf.GPUOptions(per_process_gpu_memory_fraction=0.7)
            sess = tf.Session(config=tf.ConfigProto(
                gpu_options=gpu_options, allow_soft_placement=True,
                log_device_placement=True))

            saver = tf.train.Saver()
            saver.restore(sess, get_checkpoint())

            try:
                timer = Timer()
                timer.tic()

                read_lines, n = processor.test_type(FLAGS.input_file)
                labels = processor._read_tsv(
                    os.path.join(FLAGS.checkpoint_dir, "label.txt"))
                label_dict = {}
                for label_str in labels:
                    label_dict[label_str[0]] = label_str[1]

                text_data = []
                while True:
                    # pdb.set_trace()
                    _logits_val, _prob_val, _labels_val = sess.run(
                        [logits, log_probs, one_hot_labels])
                    prob_val = _prob_val

                    if not isinstance(prob_val, np.ndarray):
                        raise ValueError("Unknown format probability result.")
                    if n != 1:
                        raise Exception("Unsupported inference text format.")

                    for i, prob_vals in enumerate(prob_val):
                        prob_result = []
                        for p_i, p_j in enumerate(prob_vals):
                            prob_result.append({
                                "label": label_dict[str(p_i)],
                                "probability": round(float(p_j.item()), 4)
                            })

                        prob_result = sorted(
                            prob_result,
                            key=lambda x: x['probability'],
                            reverse=True)[0:5]

                        prob_result = [
                            (p['label'], p['probability']) for p in prob_result
                        ]

                        if with_original_content:
                            text_data.append({
                                "index": line_num,
                                "text": read_lines[i][0],
                                "probability": prob_result
                            })
                        else:
                            text_data.append({
                                "index": line_num,
                                "probability": prob_result
                            })
                        line_num += 1

                    if float(0) < len(read_lines) / len(prob_val) <= float(1):
                        timer.toc()
                        return {
                            "duration": int(round(1000 * timer.total_time)),
                            "data": text_data
                        }
                    read_lines = read_lines[len(prob_val):]
            except Exception:
                tf.errors.OutOfRangeError


def get_checkpoint():
    if tf.gfile.IsDirectory(FLAGS.checkpoint_dir):
        checkpoint_dir = tf.train.latest_checkpoint(FLAGS.checkpoint_dir)
    else:
        checkpoint_dir = FLAGS.checkpoint_dir

    return checkpoint_dir


if __name__ == '__main__':
    tf.app.run()
