import os
import sys
import shutil
import tensorflow.compat.v1 as tf
tf.disable_v2_behavior()
from tensorflow.python.tools import freeze_graph

sys.path.append('/opt/letrain/engine')

from nets import nets_factory


def load_labels(label_file):
    labels_dict = {}
    with open(label_file) as f:
        for line in f:
            key, value = line.rstrip('\n').split(':')
            labels_dict[key] = value
    return labels_dict


class ModelConverter(object):

    def __init__(self, model_name, checkpoint_dir, output_path):
        self.input_node_name = "lico_input_node"
        self.output_node_name = "lico_output_node"
        self.frozen_model_name = model_name + "_frozen_model.pb"
        self.checkpoint_dir = checkpoint_dir
        self.model_name = model_name
        self.output_path = output_path
        self.tmp_dir = os.path.join(self.output_path, 'tmp')

    def _prepare(self):
        self.labels_dict = load_labels(
            os.path.join(self.checkpoint_dir, 'labels.txt')
        )
        num_classes = len(self.labels_dict)
        self.network_fn = nets_factory.get_network_fn(
            self.model_name, num_classes, is_training=False)
        self.image_size = self.network_fn.default_image_size

    def convert_ckpt_to_frozenpb(self):
        self._prepare()
        ckpt_path = tf.train.latest_checkpoint(self.checkpoint_dir)
        if not ckpt_path:
            print("[CONVERT_CHECKPOINT_ERROR]: cannot find any checkpoint "
                  "files under model path.")
            exit(1)

        tf.reset_default_graph()
        image_place = tf.placeholder(
            tf.float32, [None, self.image_size, self.image_size, 3],
            name=self.input_node_name
        )
        logits, _ = self.network_fn(image_place)
        tf.identity(logits, name=self.output_node_name)

        frozen_pb = os.path.join(
            self.tmp_dir,
            self.frozen_model_name
        )

        with tf.Session() as sess:
            # Save the graph and generate model.pb file in the output path
            # model.pb file will be given as input_graph to the
            # freeze_graph function
            tf.io.write_graph(sess.graph_def, self.tmp_dir, 'model.pb')

            freeze_graph.freeze_graph(
                input_graph=os.path.join(self.tmp_dir, 'model.pb'),
                input_saver='',
                input_binary=False,
                input_checkpoint=ckpt_path,
                output_node_names=self.output_node_name,
                restore_op_name='save/restore_all',
                filename_tensor_name='save/Const:0',
                output_graph=frozen_pb,
                clear_devices=True,
                initializer_nodes=''
            )
        print(f"[INFO] Save the frozen model to {frozen_pb}")

        # Copy label.txt to tmp_dir
        shutil.copy(os.path.join(self.checkpoint_dir, 'labels.txt'),
                    self.tmp_dir)

        # Delete model.pb file
        model_file = os.path.join(self.tmp_dir, 'model.pb')
        if os.path.exists(model_file):
            os.remove(model_file)


if __name__ == "__main__":
    tf.app.flags.DEFINE_string('checkpoint_dir', '',
                               'Path to the checkpoint directory')
    tf.app.flags.DEFINE_string('model_name', '', 'model name')
    tf.app.flags.DEFINE_string('output_path', '.',
                               'store path of output model')

    FLAGS = tf.app.flags.FLAGS

    print(f"[INFO] Params: \n"
          f"checkpoint_dir: {FLAGS.checkpoint_dir} \n"
          f"model_name: {FLAGS.model_name} \n"
          f"output_path: {FLAGS.output_path}", flush=True)

    converter = ModelConverter(FLAGS.model_name, FLAGS.checkpoint_dir,
                               FLAGS.output_path)

    converter.convert_ckpt_to_frozenpb()
