# Copyright 2021-present Lenovo
# Confidential and Proprietary

import argparse
import os
from subprocess import check_call

from trt_service_runner import TRTServiceRunner


def create_app():
    import falcon

    api = falcon.API()

    api.req_options.strip_url_path_trailing_slash = True

    model_name = os.environ.get('LETRAIN_MODEL_NAME', '')
    model_path = os.environ.get('LICO_MODEL_PATH', '')

    service_url = os.environ.get('SERVICE_URL', '')
    api.add_route(
        service_url,
        TRTServiceRunner(model_path, model_name)
    )

    return api


if __name__ == 'trt_service':
    inf_app = create_app()


if __name__ == '__main__':

    parser = argparse.ArgumentParser()

    parser.add_argument("--url", default='/api/service')
    parser.add_argument("--port", type=int, default=80)

    parser.add_argument("-n", "--model_name", required=True,
                        help="Original model topology.")
    parser.add_argument("-m", "--model_path", required=True,
                        help="Original model topology.")

    args = parser.parse_args()

    cmd = [
        'gunicorn',
        '--workers',
        '1',
        '--bind',
        '0.0.0.0:{0}'.format(args.port),
        '--pythonpath',
        os.path.dirname(__file__),
        '--access-logfile',
        '-',
        '--error-logfile',
        '-',
        '--timeout',
        '600',
        '--env',
        'LETRAIN_MODEL_NAME={0}'.format(args.model_name),
        '--env',
        'LICO_MODEL_PATH={0}'.format(args.model_path),
        '--env',
        'SERVICE_URL={0}'.format(args.url),
        'trt_service:inf_app'
    ]
    check_call(cmd)
