# Copyright 2021-present Lenovo
# Confidential and Proprietary

import base64
import sys
import traceback
from tempfile import NamedTemporaryFile

import falcon
from falcon.media.validators import jsonschema
from trt_inference import TRTDeploySer


def get_infe_serv(model_path, model_name):
    print('Init Load Model')
    inference_service_mgt = TRTDeploySer()
    inference_service_mgt.load_model(model_path, model_name)
    print('Load Model Success')
    return inference_service_mgt


class TRTServiceRunner(object):
    def __init__(self, model_path, model_name):
        self.InferenceServiceMgt = get_infe_serv(
            model_path, model_name
        )
        sys.stdout.flush()

    @jsonschema.validate(
        {
            "type": "object",
            "properties": {
                "image": {"type": "string"},
                "input_type": {
                    "type": "string",
                    "enum": ["BASE64"]
                },
            },
            "required": ["image", "input_type"]
        }
    )
    def on_post(self, req, resp):
        try:
            content = base64.b64decode(req.media.get('image'))
            with NamedTemporaryFile(mode='wb') as buf:
                buf.write(content)
                buf.flush()
                inf_result = self.InferenceServiceMgt.inference(buf.name)
        except ValueError:
            traceback.print_exc()
            raise falcon.HTTPBadRequest(
                description='The format of image is unsupported'
            )
        except TypeError:
            traceback.print_exc()
            raise falcon.HTTPBadRequest(
                description='It is not an image'
            )
        except Exception:
            traceback.print_exc()
            raise falcon.HTTP_INTERNAL_SERVER_ERROR(
                description='Internal error of inference server'
            )
        resp.media = inf_result
