# Copyright 2022-present Lenovo
# Confidential and Proprietary
import os
import shutil
import zipfile
import tensorflow.compat.v1 as tf
tf.disable_v2_behavior()
from uuid import uuid4
from datetime import datetime

FLAGS = tf.app.flags.FLAGS


class BaseExportService(object):
    def __init__(self):
        self.pre_check()

    def pre_check(self):
        '''Pre check all variables to make sure following work can be executed.
        '''
        if not FLAGS.lico_checkpoint_dir:
            raise ValueError("Checkpoint dir must be provided to restore "
                             "model.")
        if not FLAGS.lico_pb_path:
            raise ValueError("Pb dir must be provided to restore model.")
        self.tmp_dir = os.path.join(FLAGS.lico_pb_path, f'tmp_{uuid4()}')

    def get_export_path(self, export_path_base, version_control, version):
        if version_control:
            count = version
            modify = False
            if not os.path.exists(export_path_base):
                os.mkdir(export_path_base)
            else:
                for v in os.listdir(export_path_base):
                    if os.path.isdir(os.path.join(export_path_base, v)) and \
                            v.isdigit() and int(v) >= count:
                        count = int(v)
                        modify = True
                if modify:
                    count += 1
            export_path = os.path.join(
                tf.compat.as_bytes(export_path_base),
                tf.compat.as_bytes(str(count)))
        else:
            if not os.path.exists(export_path_base):
                os.mkdir(export_path_base)
            export_path = os.path.join(
                tf.compat.as_bytes(export_path_base),
                tf.compat.as_bytes(str(1)))
            if os.path.exists(export_path):
                import shutil
                shutil.rmtree(export_path)
                print("Export directory {} has been cleared.".format(
                    export_path))
        return export_path

    def checkpoint2pb(self):
        pass

    def package_model_into_zip_file(self, export_path_base, export_path):
        # copy files which need to be packaged to tmp dir
        os.makedirs(self.tmp_dir)
        try:
            shutil.move(export_path, os.path.join(self.tmp_dir, '1'))
            print("[INFO] Move model files to package directory.")
            # package model files
            zip_name = f'model_TFServing_{int(datetime.now().timestamp())}.zip'
            with zipfile.ZipFile(
                    os.path.join(export_path_base, zip_name),
                    'w',
                    zipfile.ZIP_DEFLATED
            ) as zf:
                print("[INFO] Start packaging.")
                for path, dirnames, filenames in os.walk(self.tmp_dir):
                    for filename in filenames:
                        zf.write(
                            os.path.join(path, filename),
                            os.path.join(path.replace(self.tmp_dir, ''),
                                         filename)
                        )
            print(f"[INFO] Finished. You can download zip file from "
                  f"{os.path.join(export_path_base, zip_name)}")
        finally:
            shutil.rmtree(self.tmp_dir)
