# Copyright 2022-present Lenovo
# Confidential and Proprietary
import os
import sys
from shutil import copyfile

import tensorflow.compat.v1 as tf
tf.disable_v2_behavior()
from tensorflow.python.platform import gfile
from export_model import BaseExportService

parentpath = '/opt/letrain/'
path_list = [parentpath,
             parentpath + 'lib',
             parentpath + 'engine',
             parentpath + 'inference',
             parentpath + 'applications',
             os.path.join(parentpath, "applications/frcnn"),
             ]
sys.path += path_list

tf.app.flags.DEFINE_string('demo_net', 'VGGnet_test', 'Network to use [vgg16]')
tf.app.flags.DEFINE_string('category_file', '', 'category file')
tf.app.flags.DEFINE_string('model_path', None, 'model_path')
tf.app.flags.DEFINE_integer('frcnn_num_classes', 21, 'number of classes')

FLAGS = tf.app.flags.FLAGS
FLAGS.model_path = FLAGS.lico_checkpoint_dir

import flags4
flags4.DEFINE_boolean('use_cpu', False, 'Use cpu.')

from lib.networks.factory import get_network
import tf_slim as slim

LABELS_FILE = os.path.join(FLAGS.lico_checkpoint_dir, 'category.txt')
VERSIONCONTROLL = FLAGS.lico_version_control


class FasterRCNNExportService(BaseExportService):
    def __init__(self):
        self.pre_check()
        if not gfile.Exists(LABELS_FILE):
            raise ValueError("label file: %s is not found." % LABELS_FILE)

    def _read_category_file(self):
        global CLASSES
        if FLAGS.category_file:
            category_file = FLAGS.category_file
        else:
            category_file = os.path.join(FLAGS.model_path, "category.txt")
        _classes_auto = []
        with open(category_file) as f:
            lines = f.readlines()
        for line in lines:
            line = line.lower().strip('\n')
            _classes_auto.append(line)
        CLASSES = tuple(_classes_auto)
        FLAGS.frcnn_num_classes = len(CLASSES)

    def save_pb(self, sess, export_path_base, graph):
        export_path = self.get_export_path(
            export_path_base,
            VERSIONCONTROLL,
            FLAGS.model_version
        )

        builder = tf.saved_model.builder.SavedModelBuilder(export_path)

        Placeholder = tf.saved_model.utils.build_tensor_info(
            graph.get_tensor_by_name("Placeholder:0")
        )
        Placeholder_1 = tf.saved_model.utils.build_tensor_info(
            graph.get_tensor_by_name("Placeholder_1:0")
        )

        cls_prob = tf.saved_model.utils.build_tensor_info(
            graph.get_tensor_by_name("cls_prob:0")
        )
        bbox_pred = tf.saved_model.utils.build_tensor_info(
            graph.get_tensor_by_name('bbox_pred/bbox_pred:0')
        )
        rois = tf.saved_model.utils.build_tensor_info(
            graph.get_tensor_by_name('rois:0')
        )
        cls_score = tf.saved_model.utils.build_tensor_info(
            graph.get_tensor_by_name('cls_score/cls_score:0')
        )

        labeling_signature = (
            tf.saved_model.signature_def_utils.build_signature_def(
                inputs={
                    "Placeholder": Placeholder,
                    "Placeholder_1": Placeholder_1

                },
                outputs={
                    "cls_prob": cls_prob,
                    "bbox_pred": bbox_pred,
                    "rois": rois,
                    "cls_score": cls_score
                },
                method_name=tf.saved_model.signature_constants
                .PREDICT_METHOD_NAME
            )
        )
        legacy_init_op = tf.group(
            tf.tables_initializer(),
            name='legacy_init_op'
        )
        builder.add_meta_graph_and_variables(
            sess,
            [tf.saved_model.tag_constants.SERVING],
            signature_def_map={'predict': labeling_signature},
            legacy_init_op=legacy_init_op,
            clear_devices=True
        )

        builder.save()
        DESTINATION = export_path.decode() + '/category.txt'
        copyfile(LABELS_FILE, DESTINATION)
        self.package_model_into_zip_file(export_path_base, export_path)

    def checkpoint2pb(self):
        graph = tf.Graph()
        with graph.as_default():
            self._read_category_file()
            model = get_network(FLAGS.demo_net)
            print(model)

            ckpt_path = tf.train.latest_checkpoint(FLAGS.lico_checkpoint_dir)

            with tf.Session(config=tf.ConfigProto(
                    allow_soft_placement=True)) as sess:
                init = tf.global_variables_initializer()
                sess.run(init)
                # Restore variables from checkpoint file into graph
                variables_to_restore = slim.get_variables_to_restore()
                saver = tf.train.Saver(variables_to_restore)
                saver.restore(sess, ckpt_path)
                self.save_pb(sess, FLAGS.lico_pb_path, graph)


if __name__ == '__main__':
    od_obj = FasterRCNNExportService()
    od_obj.checkpoint2pb()
