# Copyright 2022-present Lenovo
# Confidential and Proprietary
import os
import sys
from shutil import copyfile

import tensorflow.compat.v1 as tf
tf.disable_v2_behavior()
from export_model import BaseExportService
from tensorflow.python.eager import context
from tensorflow.python.platform import gfile

parentpath = '/opt/letrain/'
path_list = [parentpath,
             parentpath + 'lib',
             parentpath + 'engine',
             parentpath + 'inference',
             parentpath + 'applications',
             os.path.join(parentpath, "applications/frcnn")]
sys.path += path_list

from nets import nets_factory

FLAGS = tf.app.flags.FLAGS

import tf_slim as slim  # noqa: F401
tf.logging.set_verbosity(tf.logging.INFO)
LABELS_FILE = os.path.join(FLAGS.lico_checkpoint_dir, 'labels.txt')
VERSIONCONTROLL = FLAGS.lico_version_control


class ExportService(BaseExportService):
    def __init__(self):
        self.pre_check()
        if not FLAGS.lico_model_name:
            raise ValueError("Model name must be provided.")
        if not gfile.Exists(LABELS_FILE):
            raise ValueError("label file: %s is not found." % LABELS_FILE)

    def save_pb(self, sess, export_path_base, graph):
        export_path = self.get_export_path(
            export_path_base,
            VERSIONCONTROLL,
            FLAGS.model_version
        )

        builder = tf.saved_model.builder.SavedModelBuilder(export_path)
        image_inputs = tf.saved_model.utils.build_tensor_info(
            graph.get_tensor_by_name("Placeholder:0")
        )
        logits = tf.saved_model.utils.build_tensor_info(
            graph.get_tensor_by_name("Softmax:0")
        )
        labeling_signature = (
            tf.saved_model.signature_def_utils.build_signature_def(
                inputs={"inputs": image_inputs},
                outputs={"logits": logits},
                method_name=tf.saved_model.signature_constants
                .PREDICT_METHOD_NAME
            )
        )
        legacy_init_op = tf.group(
            tf.tables_initializer(),
            name='legacy_init_op'
        )
        builder.add_meta_graph_and_variables(
            sess,
            [tf.saved_model.tag_constants.SERVING],
            signature_def_map={'predict': labeling_signature},
            legacy_init_op=legacy_init_op,
            clear_devices=True
        )

        builder.save()
        DESTINATION = export_path.decode() + '/labels.txt'
        copyfile(LABELS_FILE, DESTINATION)
        sys.stdout.flush()
        self.package_model_into_zip_file(export_path_base, export_path)

    def load(self, saver, sess, ckpt_path):
        '''Load trained weights.

        Args:
          saver: TensorFlow saver object.
          sess: TensorFlow session.
          ckpt_path: path to checkpoint file with parameters.
        '''
        saver.restore(sess, ckpt_path)
        print(("Restored model parameters from {}".format(ckpt_path)))

    def get_available_device(self):
        num_gpu = context.num_gpus()
        return '/device:GPU:0' if num_gpu else '/device:CPU:0'

    def checkpoint2pb(self):
        graph = tf.Graph()
        with graph.as_default():
            # Get target label doc for matching probability
            labels_dict = {}
            try:
                with open(LABELS_FILE) as f:
                    for line in f:
                        key, value = line.rstrip('\n').split(':')
                        labels_dict[key] = value
            except IOError:
                raise ValueError("Can't open file:", LABELS_FILE)

            infer_device = self.get_available_device()
            with tf.device(infer_device):
                # Get model
                num_classes = len(labels_dict)
                network_fn = nets_factory.get_network_fn(
                    FLAGS.lico_model_name,
                    num_classes,
                    is_training=False
                )
                image_size = network_fn.default_image_size

                def _pre_tf_process(feature):
                    try:
                        image_decoded = tf.image.convert_image_dtype(
                            feature,
                            tf.float32
                        )
                        image_decoded = tf.image.resize_images(
                            image_decoded,
                            size=[image_size, image_size]
                        )
                        image_final = tf.map_fn(
                            lambda x: tf.image.per_image_standardization(x),
                            image_decoded)
                        return image_final
                    except Exception as e:
                        print(e)

                image_tensor = tf.placeholder(
                    dtype=tf.float32,
                    shape=[None, None, None, None]
                )
                image = _pre_tf_process(image_tensor)
                image.set_shape([None, image_size, image_size, 3])
                image = tf.reshape(image, [-1, image_size, image_size, 3])
                logits, _ = network_fn(image)

                # Calculate probabilities with softmax activation
                prob = tf.nn.softmax(logits)
                print(prob)

                # Create session
                gpu_options = tf.GPUOptions(per_process_gpu_memory_fraction=0.7)
                sess = tf.Session(
                    config=tf.ConfigProto(
                        gpu_options=gpu_options,
                        allow_soft_placement=True,
                        log_device_placement=True
                    )
                )

                # Restore variables from checkpoint file into graph
                saver = tf.train.Saver()
                ckpt = tf.train.latest_checkpoint(FLAGS.lico_checkpoint_dir)
                self.load(saver, sess, ckpt)
                self.save_pb(sess, FLAGS.lico_pb_path, graph)


if __name__ == '__main__':
    class_obj = ExportService()
    class_obj.checkpoint2pb()
